/*
 * MoleculeOfAtomAction.cpp
 *
 *  Created on: May 12, 2010
 *      Author: heber
 */

#include "Helpers/MemDebug.hpp"

#include "Actions/SelectionAction/MoleculeOfAtomAction.hpp"
#include "Actions/ActionRegistry.hpp"
#include "Descriptors/MoleculeDescriptor.hpp"
#include "atom.hpp"
#include "Helpers/Log.hpp"
#include "Helpers/Verbose.hpp"
#include "World.hpp"

#include <iostream>
#include <string>

using namespace std;

#include "UIElements/UIFactory.hpp"
#include "UIElements/Dialog.hpp"
#include "Actions/ValueStorage.hpp"


// memento to remember the state when undoing

class SelectionMoleculeOfAtomState : public ActionState {
public:
  SelectionMoleculeOfAtomState(std::vector<molecule*> selectedMolecules, atom *_Walker) :
    selectedMolecules(selectedMolecules),
    Walker(_Walker)
  {}
  std::vector<molecule*> selectedMolecules;
  atom *Walker;
};

const char SelectionMoleculeOfAtomAction::NAME[] = "select-molecule-of-atom";

SelectionMoleculeOfAtomAction::SelectionMoleculeOfAtomAction() :
  Action(NAME)
{}

SelectionMoleculeOfAtomAction::~SelectionMoleculeOfAtomAction()
{}

void SelectionMoleculeOfAtom() {
  ActionRegistry::getInstance().getActionByName(SelectionMoleculeOfAtomAction::NAME)->call(Action::NonInteractive);
};

Dialog* SelectionMoleculeOfAtomAction::fillDialog(Dialog *dialog) {
  ASSERT(dialog,"No Dialog given when filling action dialog");

  dialog->queryAtom(NAME, ValueStorage::getInstance().getDescription(NAME));

  return dialog;
}

Action::state_ptr SelectionMoleculeOfAtomAction::performCall() {
  atom *Walker = NULL;

  ValueStorage::getInstance().queryCurrentValue(NAME, Walker);

  std::vector<molecule *> selectedMolecules = World::getInstance().getSelectedMolecules();
  DoLog(1) && (Log() << Verbose(1) << "Selecting molecule to which " << Walker->getName() << " belongs." << endl);
  World::getInstance().selectMoleculeOfAtom(Walker);
  return Action::state_ptr(new SelectionMoleculeOfAtomState(selectedMolecules, Walker));
}

Action::state_ptr SelectionMoleculeOfAtomAction::performUndo(Action::state_ptr _state) {
  SelectionMoleculeOfAtomState *state = assert_cast<SelectionMoleculeOfAtomState*>(_state.get());

  World::getInstance().clearMoleculeSelection();
  for(std::vector<molecule *>::iterator iter = state->selectedMolecules.begin(); iter != state->selectedMolecules.end(); ++iter)
    World::getInstance().selectMolecule(*iter);

  return Action::state_ptr(new SelectionMoleculeOfAtomState(state->selectedMolecules, state->Walker));
}

Action::state_ptr SelectionMoleculeOfAtomAction::performRedo(Action::state_ptr _state){
  SelectionMoleculeOfAtomState *state = assert_cast<SelectionMoleculeOfAtomState*>(_state.get());

  World::getInstance().selectMoleculeOfAtom(state->Walker);

  return Action::state_ptr(new SelectionMoleculeOfAtomState(state->selectedMolecules, state->Walker));
}

bool SelectionMoleculeOfAtomAction::canUndo() {
  return true;
}

bool SelectionMoleculeOfAtomAction::shouldUndo() {
  return true;
}

const string SelectionMoleculeOfAtomAction::getName() {
  return NAME;
}
